@@ -5,7 +5,7 @@ use Mojolicious::Plugin::Vparam::Common qw(:all);
 use version;
 use List::MoreUtils qw(firstval natatime mesh);
 
-our $VERSION    = '2.04';
+our $VERSION    = '2.05';
 
 # Regext for shortcut parser
 our $SHORTCUT_REGEXP = qr{
@@ -44,6 +44,7 @@ sub register {
     $conf->{time}           = '%T'          unless exists $conf->{time};
     $conf->{datetime}       = '%F %T %z'    unless exists $conf->{datetime};
 
+    $conf->{blessed}        //= 1;
     $conf->{optional}       //= 0;
     $conf->{skipundef}      //= 0;
     $conf->{multiline}      //= 0;
@@ -175,6 +176,10 @@ sub register {
             ? delete $params{-multiline}
             : $conf->{multiline}
         ;
+        my $blessed = exists $params{-blessed}
+            ? delete $params{-blessed}
+            : $conf->{blessed}
+        ;
 
         # Internal variables
         my $vars = $self->stash->{'vparam-vars'} //= {};
@@ -212,6 +217,7 @@ sub register {
             $attr{optional}     //= $optional;
             $attr{skipundef}    //= $skipundef;
             $attr{multiline}    //= $multiline;
+            $attr{blessed}      //= $blessed;
 
             # Apply type
             if( defined( my $type = $attr{type} ) ) {
@@ -271,7 +277,10 @@ sub register {
                 if( $vars->{json} ) {
                     $vars->{pointer} //=
                         Mojo::JSON::Pointer->new( $vars->{json} );
-                    @input = $vars->{pointer}->get( $attr{jpath} );
+                    if( $vars->{pointer}->contains( $attr{jpath} ) ) {
+                        my $value = $vars->{pointer}->get( $attr{jpath} );
+                        @input = 'ARRAY' eq ref $value ? @$value : $value;
+                    }
                 }
             } elsif( $attr{cpath} ) {
                 # CSS
@@ -353,11 +362,11 @@ sub register {
                 }
 
                 # Apply pre filter
-                $out = $attr{pre}->( $self, $out )    if $attr{pre};
+                $out = $attr{pre}->( $self, $out, \%attr )   if $attr{pre};
 
                 # Apply validator
                 if( $attr{valid} ) {
-                    if( my $error = $attr{valid}->($self, $out)  ) {
+                    if( my $error = $attr{valid}->($self, $out, \%attr) ) {
                         # Set default value if error
                         $out = $attr{default};
 
@@ -384,10 +393,12 @@ sub register {
                 # Hack for bool values:
                 # HTML forms do not transmit if checkbox off
                 $out = $attr{default}
-                    if $attr{type} && $attr{type} eq 'bool' and not defined $in;
+                    if      $attr{type}
+                        and $attr{type} =~ m{^(?:bool|true|checkbox)$}
+                        and not defined $in;
 
                 # Apply post filter
-                $out = $attr{post}->( $self, $out )   if $attr{post};
+                $out = $attr{post}->( $self, $out, \%attr )  if $attr{post};
 
                 # Apply other filters
                 for my $key ( keys %attr ) {
@@ -813,6 +824,7 @@ Just set this parameters as HashRef with new types definition.
 You can simple add you own filters.
 Just set this parameters as HashRef with new filters definition.
 
+
 =item vsort_page
 
 Parameter name for current page number in I<vsort>. Default: page.
@@ -864,6 +876,11 @@ if no format specified, return L<DateTime> object.
 Datetime format for strftime. Default: '%F %T %z'.
 if no format specified, return L<DateTime> object.
 
+=item blessed
+
+By default return objects used for parse or validation:
+L<Mojo::URL>, L<DateTime>, etc.
+
 =item optional
 
 By default all parameters are required. You can change this by set this
@@ -1012,6 +1029,10 @@ I<TRUE> can be 1, yes, true, ok
 
 I<FALSE> can be 0, no, false, fail
 
+=item
+
+Empty string is I<FALSE>
+
 =back
 
 Other values get error.
@@ -1275,6 +1296,14 @@ You can simple split I<textarea> to values:
 
 Empty lines ignored.
 
+=head2 blessed
+
+Keep and return blessed object for parsed parameters if available.
+Vparam always return scalars if disabled.
+
+Note: if defined I<date>, I<time>, I<datetime> then always return
+formatted scalar.
+
 =head2 jpath
 
 If you POST data not form but raw JSON you can use JSON Pointer selectors
@@ -1289,6 +1318,8 @@ from L<Mojo::JSON::Pointer> to get and validate parameters.
         lat     => { type => 'lat', jpath => '/point/lat' },
     );
 
+Note: we don`t support multikey in json. Use hash or die.
+
 =head2 cpath
 
 Same as jpath but parse XML/HTML using CSS selectors from L<Mojo::DOM::CSS>.
